
## ----echo=FALSE----------------------------------------------------------
results = "hide"; echo = FALSE


## ----setup, include=FALSE, cache=FALSE-----------------------------------
library(knitr)
library(methods)
options(replace.assign=FALSE,width=50)

opts_chunk$set(fig.path='knitr_figure/graphics-', 
               cache.path='knitr_cache/graphics-', 
               fig.align='center', 
               dev='pdf', fig.width=5, fig.height=5, 
               fig.show='hold', cache=FALSE, par=TRUE)
knit_hooks$set(crop=hook_pdfcrop)

knit_hooks$set(par=function(before, options, envir){
  if (before && options$fig.show!='none') {
    par(mar=c(3,3,2,1),cex.lab=.95,cex.axis=.9,
        mgp=c(2,.7,0),tcl=-.01, las=1)
  }}, crop=hook_pdfcrop)


## ----echo=echo, results=results------------------------------------------
methods("mean")


## ----echo=echo, results="hide"-------------------------------------------
body("mean.default")


## ----echo=echo, results=results------------------------------------------
args("mean")


## ----echo=echo, results='hide'-------------------------------------------
mean.cohort = function(x, ...) {
  m1 = mean(x$details[ ,1], ...)
  m2 = mean(x$details[ ,2], ...)
  return(c(m1, m2))
}


## ----echo=echo, results=results------------------------------------------
sd = function(x, ...) UseMehod("sd")
sd.default = function(x, ...) stats::sd(x, ...)
sd.cohort = function(x, ...) {
  s1 = sd(x$details[ ,1], ...)
  s2 = sd(x$details[ ,2], ...)
  return(c(s1, s2))
} 


## ----echo=echo, results=results------------------------------------------
isGeneric("mean")
setGeneric("mean")


## ----echo=echo, results=results, message=FALSE---------------------------
setMethod("mean", signature=c("Cohort"), 
          definition=function(x, ...) {
            m1 = mean(x@details[ ,1], ...)
            m2 = mean(x@details[ ,2], ...)
            return(c(m1, m2))
      }
)


## ----echo=echo, results=results, message=FALSE---------------------------
isGeneric("sd")
setGeneric("sd")
setMethod("sd", signature=c("Cohort"), 
          definition=function(x, na.rm=FALSE) {
            m1 = sd(x@details[ ,1], na.rm=na.rm)
            m2 = sd(x@details[ ,2], na.rm=na.rm)
            return(c(m1, m2))
      }
)


## ----P2, echo=FALSE------------------------------------------------------
randu = setRefClass("randu", 
    fields = list(calls = "numeric", seed="numeric", state="numeric"))
randu$methods(get_state = function() state)
randu$methods(set_state = function(initial) state <<- initial)
randu$methods(get_seed = function() seed)
randu$methods(get_num_calls = function() calls)
randu$methods(r = function() {
  calls <<- calls + 1
  state <<- (65539*state) %% 2^31
  return(state/2^31)
})


## ------------------------------------------------------------------------
r = randu(calls=0, seed=10, state=10)
r$r()
r$get_state()
r$get_seed()


## ----echo=echo-----------------------------------------------------------
##Solutions - see below


## ------------------------------------------------------------------------
r = randu(calls=0, seed=10, state=10)
r$get_num_calls()
r$r()
r$r()
r$get_num_calls()


## ----eval=FALSE----------------------------------------------------------
## library("nclRadvanced")
## vignette("solutions2", package="nclRadvanced")


